/*
 * file CqResultSet.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqResultSet
 *
 *  Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import java.util.Iterator;

import com.ibm.rational.wvcm.stp.StpReleasable;
import com.ibm.rational.wvcm.stp.cq.CqQuery.DisplayField;
import com.ibm.rational.wvcm.stp.cq.CqQuery.DisplayField.FieldType;


/**
 * An interface specifying the structure returned by CqQuery.doExecute and
 * CqRecordType.doQuery to represent the results of executing a ClearQuest
 * query.
 * <p>
 * A CqResultSet is an Iterable as well as an Iterator. Each invocation of its
 * iterator() method <i>does not</i> restart the iteration. It just continues
 * from the last read row. That is CqResultSet.iterator() simply returns the
 * CqResultSet object. It is provided to allow the use of CqResultSet objects in
 * the Java 5 for-each construct.
 * 
 * @see CqQuery#doExecute(long, long,
 *      com.ibm.rational.wvcm.stp.cq.CqQuery.ListOptions,
 *      com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf[])
 * @see CqRecordType#doQuery(String, long, long,
 *      com.ibm.rational.wvcm.stp.cq.CqQuery.ListOptions)
 * @see CqRecordType#doQuery(com.ibm.rational.wvcm.stp.cq.CqQuery.DisplayField[],
 *      com.ibm.rational.wvcm.stp.cq.CqQuery.Filter, long, long,
 *      com.ibm.rational.wvcm.stp.cq.CqQuery.ListOptions)
 */
public interface CqResultSet extends StpReleasable, Iterator<CqRowData>,
                Iterable<CqRowData>
{
    /**
     * @return A String[] containing the label defined for each column of the
     * result set.
     */
    String[] getColumnLabels();

    /**
     * @return A FieldType[] containing FieldType enumerators that specify the
     *         type of data returned in each column of the result set. In the
     *         result set generated by a raw SQL query the column type is
     *         inferred from the database data type and not a field definition.
     *         Thus, in this case, only the following generic FieldType
     *         enumerators are used in this array: BINARY, SHORT_STRING,
     *         MULTILINE_STRING, INTEGER, FLOAT, and DATE_TIME.
     */
    FieldType[] getColumnTypes();

    /**
     * @return A String containing the query expressed as a vendor-specific SQL
     *         select statement.
     */
    String getSql();

    /**
     * The total number of rows found by the query in the database. This value
     * is available only if requested when the query was executed and this
     * result set was generated. If computed, this number would be the upper
     * bound on the number of CqRowData elements to expect in this result set.
     * The actual content of the iterator may be less or even empty depending
     * on the options specified for the execution of the query.
     * 
     * @return If a row count was requested, the total number of rows found by
     * the query in the database; if a row count was not requested, -1.
     * 
     * @see CqQuery.ListOptions#getEnableRowCount()
     */
    long getRowCount();

    /**
     * @return The absolute upper bound on the maximum row number that can be
     * returned by a query. This value is established by the ClearQuest database
     * administrator and cannot be changed via this API.
     */
    long getRowNumberHardLimit();

    /**
     * @return The default upper bound on the maximum row number that can be
     *         returned by a query. This value is established by the ClearQuest
     *         database administrator. It can be overridden by defining
     *         {@link CqQuery.CommonOptions#getRowNumberLimit()} to return the
     *         overriding value.
     */
    long getRowNumberSoftLimit();

    /**
     * @return Answers whether or not a row number generated by the query
     *         exceeded the smaller of
     *         <ul>
     *         <li><code>ListOption.getRowNumberLimit()</code> (which
     *         defaults to <code>getRowNumberSoftLimit()</code>)
     *         <li><code>getRowNumberHardLimit()</code>
     *         </ul>. Note that the value returned by this method does not
     *         indicate whether or not the <code>maxRows</code> limit was
     *         exceeded.
     * 
     * @see CqQuery.ListOptions#getRowNumberLimit()
     */
    boolean isRowNumberLimitExceeded();

    /**
     * Returns a CqQuery proxy for the query that was executed to generate this
     * result set. It is available only from CqQuery.doExecute and only if the
     * ListOptions passed to that method defined
     * ListOptions.getQueryPropertyRequest() to return a non-null
     * PropertyRequest.
     * 
     * @return If CqQuery.ListOptions.getQueryPropertyRequest() was not 
     *         <b>null</b>, a proxy for the executed query populated with the 
     *         requested properties; otherwise <b>null</b>.
     * 
     * @see CqQuery.ListOptions#getQueryPropertyRequest()
     */
    CqQuery getQuery();
}
